/*******************************************************************************
 * Copyright (c) 2008-2016 SWTChart project. All rights reserved. 
 * 
 * This code is distributed under the terms of the Eclipse Public License v1.0
 * which is available at http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.swtchart;

import org.eclipse.swt.graphics.RGB;

/**
 * Constant values.
 */
public class Constants {

    /** the large font size */
    final static public int LARGE_FONT_SIZE = 13;

    /** the medium font size */
    final static public int MEDIUM_FONT_SIZE = 11;

    /** the small font size */
    final static public int SMALL_FONT_SIZE = 9;

    /** the color for light blue */
    @Deprecated
    final static public RGB LIGHT_BLUE = new RGB(153, 186, 243);

    /** the color for blue */
    @Deprecated
    final static public RGB BLUE = new RGB(0, 0, 255);

    /** the color for white */
    @Deprecated
    final static public RGB WHITE = new RGB(255, 255, 255);

    /** the color for gray */
    @Deprecated
    final static public RGB GRAY = new RGB(200, 200, 200);

    /** the color for dark gray */
    @Deprecated
    final static public RGB DARK_GRAY = new RGB(150, 150, 150);

    /** the color for black */
    @Deprecated
    final static public RGB BLACK = new RGB(0, 0, 0);
}
