package org.swtchart.ext;

/**
 * Messages
 */
public class Messages {

    /** the menu group for adjust axis range menus */
    public static final String ADJUST_AXIS_RANGE_GROUP = "&Adjust Axis Range";

    /** the menu for adjust axis range */
    public static final String ADJUST_AXIS_RANGE = "&Adjust Axis Range";

    /** the menu for adjust X axis range */
    public static final String ADJUST_X_AXIS_RANGE = "Adjust &X Axis Range";

    /** the menu for adjust Y axis range */
    public static final String ADJUST_Y_AXIS_RANGE = "Adjust &Y Axis Range";

    /** the menu group for zoom in menus */
    public static final String ZOOMIN_GROUP = "Zoom &In";

    /** the menu for zoom in */
    public static final String ZOOMIN = "Zoom &In\tCtrl+Up";

    /** the menu for zoom in X axis */
    public static final String ZOOMIN_X = "Zoom In &X Axis";

    /** the menu for zoom in Y axis */
    public static final String ZOOMIN_Y = "Zoom In &Y Axis";

    /** the menu group for zoom out menus */
    public static final String ZOOMOUT_GROUP = "Zoom &Out";

    /** the menu for zoom out */
    public static final String ZOOMOUT = "Zoom &Out\tCtrl+Down";

    /** the menu for zoom out X axis */
    public static final String ZOOMOUT_X = "Zoom Out &X Axis";

    /** the menu for zoom out Y axis */
    public static final String ZOOMOUT_Y = "Zoom Out &Y Axis";

    /** the menu for save as */
    public static final String SAVE_AS = "Save As...";

    /** the menu for opening properties dialog */
    public static final String PROPERTIES = "P&roperties...";

    /** the title for save as dialog */
	public static final String SAVE_AS_DIALOG_TITLE = "Save As";

}
